##################################################
# Program for ABM of RAZ 13 project############## 
# Authors: B. Bont and K. Erdlenbruch###########
#################################################

# To be launched with Netlogo 6.0
# To be combined with abm-raz13.nlgo file

############################
#Librairies##
############################
library(rlang)
library(rJava)
library(RNetLogo)
library(dplyr)
library(tidyr)
library(ggplot2)
library(ggrepel)
library(gtable)
library(grid)
library(evd)
library(pracma)
library(stringr)


# set Working directory and use appropriate Netlogo program
wd <- "C:/2013-2020/PR-RAZ13/NetLogoModel/pmt-abm/Communication"
setwd(file.path(wd,"r-stats"))
path.Netlogo <- "C:/Program Files/NetLogo 6.0/app"
path.model <- file.path(wd,"netlogo-model","abm-raz13.nlogo")

#######################
## Instructions ######
######################

#### Modify the function "runSimulationPlan" to choose your plan
#### Then execute the commented lines below:

## 1. Load all functions 
## 2. Generate virtual population
virtualPop <- createHouseholdsDataAndNetwork()
households <- virtualPop[[1]]
edgeList <- virtualPop[[2]]

## 3. Open netlogo and load model 
# if the command does not work open and close netlogo by hand
openNLogoAndLoadModel(GUI=F)

## 4. run the simulation plan telling the filename where you save data
runSimulationPlan(filename = "base-filename")

## 5. Copy outputs files from outputs folder to data folder
# you should have then in your data folder the files: 
# "results-base-filename.csv"
# "results-base-filename-indiv.csv"
# "households-base-filename.csv"

## 6. You can load and see your results by runing the following line
#readNPlot(filename = "base-filename")




### Model Loading #####################

openNLogoAndLoadModel <- function(GUI = GUI){
  nl = NLStart(nl.path = path.Netlogo, 
             gui = GUI, 
             nl.jarname = "netlogo-6.0.0.jar")
             setwd(file.path(wd,"r-stats"))
  NLLoadModel(path.model)
  setwd(file.path(wd,"r-stats"))
}

#########################################
#### Data used #############
#########################################

surv4 <- read.table("data/survey.csv",
                    sep=";",
                    dec=".",
                    header = T)

surv4.Var = tbl_df(subset(surv4, surv4$depart=="Var"))

#########################################
#### Computation of Odds Ratios #############
#########################################

computeAndSaveORs <- function(){
modLTemp<-glm(formula = Min1TempMeasurePlanned ~ P1_2_abm + P2_2_abm +
                copingtemp + experience_abm +
                network_15, family = binomial(link = "logit"), data = surv4.Var)

SummodelTemp <- summary(modLTemp)

write.csv2(SummodelTemp$coefficients, file="outputs/Raz13model.csv")

oddsTemp <- exp(cbind(coef(modLTemp),confint(modLTemp)))

write.csv2(oddsTemp,file="../outputs/oddsRaz13.csv")
}

##############################################
### Initialisation of global variables ####
##############################################
init_or <- function(){
  oddsTemp <- read.csv2("../netlogo-model/data/oddsRaz13.csv")
  NLCommand(paste("set or-intercept", oddsTemp[1,3]))
  NLCommand(paste("set or-proba.flood", oddsTemp[2,3]))
  NLCommand(paste("set or-consequence", oddsTemp[3,3]))
  NLCommand(paste("set or-coping", oddsTemp[4,3]))
  NLCommand(paste("set or-experience", oddsTemp[5,3]))
  NLCommand(paste("set or-network", oddsTemp[6,3]))
}
#########################################
### Population generation ###############
#########################################
#We need to create two function for spatial disposal
compute_area_unique <- function(x,y) {
  l = 500 / 6
  if (x < l) return(1)
  if (x > 5*l) return(1)
  if ((x > l) & (x < 2 *l)) return(2)
  if ((x > 4*l) & (x < 5*l)) return(2)
  if (((x > 2.5*l) & (x < 3.5*l)) & (y > 400)) return(3)
  return(3)
}

compute_area <- function(vx,vy){
  return(mapply(compute_area_unique,vx,vy))
}


source("create-network-function.R")

createHouseholdsDataAndNetwork <- function() {
popSeeds <- surv4.Var %>% 
  select(INTERNUM, 
         P1_2_abm,
         P2_2_abm, 
         copingtemp,  
         experience_abm,
         tempadaptimpl) %>%
  rename(hh.id = INTERNUM, 
         proba.flood = P1_2_abm,
         consequence = P2_2_abm,
         coping = copingtemp,
         experience = experience_abm,
         adapted = tempadaptimpl)

#Creation of a 20-fold clone of survey individuals
households <- popSeeds
for (i in 1:19){
  households <- rbind(households,popSeeds)
  }
popSize <- dim(households)[1]
households$hh.id <- seq(popSize)
households$adaptation.type <- "\"temporary\""
households$wealth.indiv <- runif(popSize, min=0, max=0)
households$earnings.indiv <- c(runif(popSize*0.1,
                                     min =0, max = 990),
                               runif(popSize*0.4,
                                     min =991, max = 1870),
                               runif(popSize*0.4,
                                     min =1871, max = 3410),
                               runif(popSize*0.1, 
                                     min =3411, max = 5500)) 
#mean(households$earnings.indiv): 2130.357

#Households are situated in three zones 
households$xcor <- runif(popSize) * 500
households$ycor <- runif(popSize) * 500
households <- households %>% 
  mutate(area = compute_area(vx = xcor, vy = ycor))

edgeList <-as_tibble(createNetwork(ids = households$hh.id,
                                   x = households$xcor,
                                   y = households$ycor), .name_repair = ~c("ins","outs") )
#colnames(edgeList)<- c("ins","outs")
return(list(households, edgeList))
}

####################################
### Population setup ###############
####################################

  
createNetLogoHouseHolds <- function(dataset){
  NLCommand(paste("create-households", dim(dataset)[1],"[]"))
  NLSetAgentSet("households", dataset)
}

setupNetwork <- function(edgeList){
  colnames(edgeList) <- c("origin", "destination")
  mapply(createLink,edgeList[,1],edgeList[,2])
}
  
createLink <- function(origin, destination) {
  commands <- paste("ask households with [hh.id =",origin,
                    "][create-connection-to one-of households with [hh.id =",
                    destination,"]]")
  lapply(commands, NLCommand)
}
  
####################################
######### Simulation functions #####
####################################

generalSetup <- function(households, edgeList) {
  # Parameters that are fixed in the simulation plan are defined in the following
  NLCommand(paste("set r", 0.02)) 
  NLCommand("setup")
  init_or()
  createNetLogoHouseHolds(households)
  NLCommand("setup-1")
  #create Network
  NLDfToList(data.frame(ins = edgeList$ins, outs=edgeList$outs))
  NLCommand(paste("set nbLinks", dim(edgeList)[1]))
  NLCommand("create-network")
  wi0s <- NLReport("[(list hh.id wealth.indiv)] of households")
  wi0s <- as.data.frame(matrix(unlist(wi0s),length(wi0s),2, byrow=T))
  colnames(wi0s) <- c("id","wi0")
  return(wi0s %>% tbl_df())
  }


runSimulation <- function(nbSteps, 
                          floodseq = 1, 
                          comPol = "no-com", 
                          b = 7, 
                          N = 1) {
  NLCommand(paste0("set com-policy \"", comPol,"\""))
  NLCommand(paste("set flood-sequence", floodseq))
  NLCommand(paste("set adaptation-duration", b))
  NLCommand(paste("set N", N))
  results <- as.data.frame(t(c(0, 
                             NLReport("damage"), 
               NLReport("wealth"), 
               NLReport("propAdapted"),
               NLReport("minWi"),
               NLReport("regulcosts"))))
  colnames(results) <- c("step","damage","wealth","adaptation","minPopWi", "regulcosts")
  for (t in seq(nbSteps)) {
    NLCommand("go")
    results <- rbind(results,
                     c(t, 
                       NLReport("damage"), 
                       NLReport("wealth"), 
                       NLReport("propAdapted"),
                       NLReport("minWi"),
                       NLReport("regulcosts")))
          }
  wis <- NLReport("[(list hh.id wealth.indiv min-wealth-indiv)] of households")
  wis <- as.data.frame(matrix(unlist(wis),length(wis),3,byrow=T))
  colnames(wis) <- c("id","wiT","wiMin")
  wisall <- left_join(wis,wi0s,by="id")
  return(list(results %>% tbl_df(), wisall%>% tbl_df()))
}

simulate <- function (duration, floods, comPol) {
  generalSetup(households, edgeList)
  return(runSimulation(duration, floods, comPol))
}

quickSetup <- function(households) {
  init_or()
  NLSetAgentSet("households", households)
  NLCommand("setup-1")
}



#################################
######### Simulation plans #####
#################################
runSimulationPlan <- function (filename = "base-filename"){
res <- NULL
resIndiv <- NULL
simu <- 0
wi0s <- generalSetup(households, edgeList)
    for (comPol in c("no-com","pc-rc","td-rc")){# "pc-r","td-r")) {
      for (f in 0:5) {
        r <- 0
        for (i in 1:10) {
        quickSetup(households)
  tempRes <- runSimulation(30, f, comPol, 7, 1)
  tempResIndiv <- tempRes[[2]]
  tempRes <- tempRes[[1]]
  tempRes$floodSequence <- f
  tempRes$com.policy <- comPol
  tempRes$rep <- r
  tempResIndiv$floodSequence <- f
  tempResIndiv$com.policy <- comPol
  tempResIndiv$rep <- r
  res <- rbind(res, tempRes)
  resIndiv <- rbind(resIndiv, tempResIndiv)
  r <- r + 1
  simu <- simu + 1
  print(paste("simu ", simu, "/",6*10*3))
  }}}
res <- tbl_df(res)
resIndiv <- tbl_df(resIndiv)

res <- res %>% mutate(rep =as.factor(rep),
                      floodSequence = as.factor(floodSequence))

resIndiv <- resIndiv %>% mutate(rep =as.factor(rep),
                           floodSequence = as.factor(floodSequence))

resIndiv %>% gather(key="indicator",value = "wealth",2:3) %>%
  ggplot() + 
  geom_density(aes(color=com.policy, linetype=indicator,x=wealth))


write.table(res, 
            file.path("outputs",paste0("results-",filename,".csv")),
            sep=";",
            dec=".")

write.table(resIndiv, 
            file.path("outputs",paste0("results-",filename,"-indiv.csv")),
            sep=";",
            dec=".")

write.table(households, 
            file.path("outputs",paste0("households-",filename,".csv")),
            sep=";",
            dec=".")
}

##########################
### read and plot data ###
##########################
readNPlot <- function (filename = "base-filename") {
 
res <- read.table(file.path("data",paste0("results-",filename,".csv")),
                  sep=";",
                  dec=".",
                  header=T)

resIndiv <- read.table(file.path("data",paste0("results-",filename,"-indiv.csv")),
                  sep=";",
                  dec=".",
                  header=T)

households <- read.table(file.path("data",paste0("households-",filename,".csv")),
                       sep=";",
                       dec=".",
                       header=T)



#visualize results:
res %>% 
  mutate(floodSequence = as.factor(floodSequence)) %>%
  group_by(step,
                 floodSequence,
                 com.policy) %>%
  summarise(mW = mean(wealth),
            minW = min(wealth),
            maxW = max(wealth)) %>%
  ggplot(aes(color = com.policy, linetype = floodSequence)) +
  geom_point(aes(x = step, y=mW, shape = floodSequence), size = 2) +
  geom_line(aes(x = step, y=mW), size = 0.5) +
  geom_ribbon(aes(x = step, 
                  ymin= minW, 
                  ymax=maxW, fill = com.policy),
              alpha=0.2, size = 0.5)
}